Package: Rmfrac
Type: Package
Title: Simulation and Statistical Analysis of Multifractional Processes
Version: 1.0.0
Description: Simulation of several fractional and multifractional processes. Includes Brownian and fractional Brownian motions, bridges and Gaussian Haar-based multifractional processes (GHBMP). Implements the methods from Ayache, Olenko and Samarakoon (2026) <doi:10.1016/j.matcom.2026.01.033> for simulation of GHBMP. Estimation of Hurst functions and local fractal dimension. Clustering realisations based on the Hurst functions. Several functions to estimate and plot geometric statistics of the processes and time series. Provides a 'shiny' application for interactive use of the functions from the package.
License: GPL-3
Encoding: UTF-8
Imports: parallel, parallelly, ggplot2(>= 3.5.0), zoo, matrixStats,
        proxy, rlang, stats, foreach, doParallel, plotly, shiny,
        graphics, shinycssloaders, fields, utils
Suggests: testthat
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/Nemini-S/Rmfrac
BugReports: https://github.com/Nemini-S/Rmfrac/issues
Authors@R: c(person("Andriy", "Olenko", email = "a.olenko@latrobe.edu.au", role = "aut", comment = c(ORCID = "0000-0002-0917-7000")),
        person("Nemini", "Samarakoon", email = "neminisamarakoon95@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0000-2107-1973")))
Maintainer: Nemini Samarakoon <neminisamarakoon95@gmail.com>
NeedsCompilation: no
Packaged: 2026-04-17 01:34:21 UTC; 21381851
Author: Andriy Olenko [aut] (ORCID: <https://orcid.org/0000-0002-0917-7000>),
  Nemini Samarakoon [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-2107-1973>)
Repository: CRAN
Date/Publication: 2026-04-17 03:30:07 UTC
Built: R 4.6.0; ; 2026-04-30 15:57:07 UTC; unix
