# Generated by roxygen2: do not edit by hand

S3method(Rmfrac::plot,H_LFD)
S3method(Rmfrac::plot,hc_hurst)
S3method(Rmfrac::plot,k_hurst)
S3method(Rmfrac::plot,mp)
S3method(Rmfrac::print,hc_hurst)
S3method(Rmfrac::print,k_hurst)
S3method(autoplot,H_LFD)
S3method(autoplot,hc_hurst)
S3method(autoplot,k_hurst)
S3method(autoplot,mp)
export(Bbridge)
export(Bm)
export(FBbridge)
export(FBm)
export(FGn)
export(GHBMP)
export(H_LFD)
export(Hurst)
export(LFD)
export(RS_Index)
export(X_max)
export(X_min)
export(cov_GHBMP)
export(cross_T)
export(cross_mean)
export(cross_rate)
export(est_cov)
export(exc_Area)
export(hclust_hurst)
export(kmeans_hurst)
export(long_streak)
export(mean_streak)
export(plot_tsest)
export(shinyapp_sim)
export(sojourn)
importFrom(doParallel,registerDoParallel)
importFrom(fields,image.smooth)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,vars)
importFrom(graphics,plot)
importFrom(matrixStats,sum2)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(parallelly,availableCores)
importFrom(parallelly,makeClusterPSOCK)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(proxy,dist)
importFrom(rlang,.data)
importFrom(shiny,runApp)
importFrom(shinycssloaders,withSpinner)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,as.dendrogram)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,loess)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rect.hclust)
importFrom(stats,rnorm)
importFrom(stats,toeplitz)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(zoo,index)
importFrom(zoo,rollapply)
