% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hierarchical_clustering.R
\name{print.hc_hurst}
\alias{print.hc_hurst}
\title{Print method for "hc_hurst" class objects}
\usage{
\method{print}{hc_hurst}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"hc_hurst"}.}

\item{...}{Other arguments.}
}
\value{
Prints an object of class \code{"hc_hurst"}.
}
\description{
Prints the results of hierarchical clustering of realisations of processes.
}
\seealso{
\code{\link{hclust_hurst}}
}
